package App::WebToy::Collection::WikiPage;
use base 'Prophet::Collection';
use App::WebToy::Model::WikiPage;

use constant record_class => 'App::WebToy::Model::WikiPage';



=head1 NAME

=head1 METHODS

=head1 DESCRIPTION

=cut

=head1 METHODS

=cut


1;

